/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * mhttpd - the mini HTTP/1.0 daemon server
 **************************************************************/

#ifndef __MHTTPD_H
#define __MHTTPD_H

#include <netinet/in.h>

// server constants
#define	DEFAULT_PORT	64201
#define DEFAULT_WORKERS	15

// mhttpd version string
#define MHTTPD_VERSION	"1.2.0"

// HTTP Date Format for strftime()
#define HTTP_DATE_FMT	"%a, %d %b %Y %H:%M:%S GMT"

// function return values
enum RESPONSES {FAILURE,SUCCESS};

// server data structure
typedef struct {
    uint16_t port;        // the server port   int
    int workers;		// worker thread count
    char http_dir[1024];	// the public_html directory
} server_data_t;

// function prototypes

// mhttpd.c
short int start_server(const uint16_t, int *);
void wait_for_connections(int);
void *run_server(void *);
void *run_worker(void *);
void stop_server(int);
int main(int argc, char **argv);

// parse.c
void send_http_header(int, int, const char *, 
		      unsigned long int, const char *, time_t);
void send_error_page(int, int, const char *, const char *);
int get_next_line(const char *, char *, int);
void parse(int, const char *, const char *);

#endif

/******************************************************************************
 *  Mini HTTP Daemon 1.2.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

