/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * lapd - the little asinine proxy daemon (a simple HTTP proxy)
 **************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "proxy.h"
#include "cache.h"

cache_t *cache_create(unsigned long int limit) {
    cache_t *cache = NULL;

    if ((cache = malloc(sizeof(cache_t))) == NULL) {
	// not enough memory to create a cache structure...
	return NULL;
    }

    cache->size = 0;
    cache->limit = limit;

    if ((cache->cache = list_create()) == NULL) {
	// not enough memory for the cache's linked list structure
	free(cache);
	return NULL;
    }

    return cache;
}

void cache_destroy(cache_t *cache) {
    int loop;
    cachefile_t *file;

    if (cache != NULL) {
	for (loop = 0; loop < cache->cache->size; loop++) {
	    file = (cachefile_t *)list_get(cache->cache, loop);
	    free(file->file_map);
	    free(file);
	}

	list_destroy(cache->cache);
	free(cache);
    }

    cache = NULL;
}

short int cache_add(cache_t *cache, cachefile_t *file) {
    if (cache == NULL) {
	return FAILURE;
    }

    if ((cache->size + file->file_size) > cache->limit) {
	// not enough space in the cache...
	return FAILURE;
    }

    // update the cache size
    cache->size += file->file_size;

    // stick it onto the cache...
    list_append(cache->cache, file);

    return SUCCESS;
}

// returns the cachefile_t structure in the cache, if found
// returns NULL otherwise...
cachefile_t *cache_retrieve(cache_t *cache, const char *uri) {
    long int loop;
    cachefile_t *file = NULL, *temp = NULL;

    if ((cache == NULL) || (uri == NULL)) {
	return NULL;
    }

    for (loop = 0; loop < cache->cache->size; loop++) {
	temp = (cachefile_t *)list_get(cache->cache, loop);

	if (strcmp(temp->uri, uri) == 0) {
	    file = temp;
	    loop = cache->cache->size;
	}
    }

    return file;
}

cachefile_t *cache_get(cache_t *cache, long int index) {
    if ((index < 0) || (index > cache->cache->size)) {
	return NULL;
    }

    return (cachefile_t *)list_get(cache->cache, index);
}

void cache_remove(cache_t *cache, cachefile_t *file) {
    if ((cache == NULL) || (file == NULL)) {
	return;
    }

    // update the cache size
    cache->size -= file->file_size;
    
    // remove the file from the cache
    list_remove(cache->cache, list_index_of(cache->cache, file));

    // free the memory associated with the memory page
    free(file->file_map);
    free(file);
}

/******************************************************************************
 *  LAPD v 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/
